#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
from typing import Union

from fastapi import APIRouter, Depends, HTTPException, Request
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.ext.asyncio import AsyncSession

from core import security
from core.config.development import settings
from core.security import verify_password, hashed_password
from crud.user import crud_admin, crud_teacher, crud_student
from models.user import SysUser, Teacher, Student
from schemas.auth import (Token, AdminLoginInfo, CurrentUserProfile,
                          UserLoginInfo, LastLoginInfo, EditPwdInfo,
                          UpdatePassword, SwaggerToken)
from schemas.base import DetailMixin
from utils.depends import get_async_db, get_current_user

login_router = APIRouter()


@login_router.post("/login",
                   summary="登录-swagger专用",
                   tags=["登录认证"],
                   response_model=SwaggerToken)
async def login_doc(request: Request,
                    db: AsyncSession = Depends(get_async_db),
                    account: OAuth2PasswordRequestForm = Depends()):
    access_token = ""
    if not access_token:
        # 查询user是否存在
        if account.username.isdigit():
            user = await crud_teacher.find_one(db, {"phone": account.username})
            utype = 1
            if not user:
                user = await crud_student.find_one(db,
                                                   {"phone": account.username})
                utype = 2
        else:
            user = await crud_admin.find_one(db, {"username": account.username})
            utype = 0
        token_payload = {"sub": user.username, "utype": utype}

        # 校验密码等
        if not user:
            raise HTTPException(status_code=400, detail="账号或密码错误!")
        elif not (await verify_password(account.password, user.password)):
            raise HTTPException(status_code=400, detail="账号或密码错误!")
        else:
            # 更新用户登录IP和时间
            login_info = LastLoginInfo(last_login_ip=request.client.host,
                                       last_login_time=datetime.datetime.now())
            await crud_admin.update(db, user, login_info)

        # 生成token
        access_token_expires = datetime.timedelta(
            minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
        access_token = await security.create_token(
            token_payload, expires_delta=access_token_expires)

    return {"access_token": access_token, "token_type": "bearer"}


@login_router.post("/admin/login",
                   tags=["登录认证"],
                   response_model=Token,
                   response_model_exclude_none=True,
                   summary="管理员登录-前端用")
async def login_as_admin(account: AdminLoginInfo,
                         request: Request,
                         db: AsyncSession = Depends(get_async_db)):

    # 从 redis 中读取 token
    # token_key = f"{account.username}:access-token"
    # access_token = await redis_client.get(token_key)
    access_token = ""
    if not access_token:
        # 查询user是否存在
        user = await crud_admin.find_one(db, {"username": account.username})
        token_payload = {"sub": user.username, "utype": 0}

        # 校验密码等
        if not user:
            raise HTTPException(status_code=400, detail="账号或密码错误!")
        elif not user.is_active:
            raise HTTPException(status_code=400, detail="账号未激活!")
        elif not (await verify_password(account.password, user.password)):
            raise HTTPException(status_code=400, detail="账号或密码错误!")
        else:
            # 更新用户登录IP和时间
            login_info = LastLoginInfo(last_login_ip=request.client.host,
                                       last_login_time=datetime.datetime.now())
            await crud_admin.update(db, user, login_info)

        # 生成token
        access_token_expires = datetime.timedelta(
            minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
        access_token = await security.create_token(
            token_payload, expires_delta=access_token_expires)

        # 把token写入redis，用于校验token有效性
        # await redis_client.set(token_key,
        #                        access_token,
        #                        expire=settings.CACHE_TOKEN_EXPIRE_SECONDS)
    return {"data": {"access_token": access_token}}


@login_router.get("/admin/account",
                  tags=["登录认证"],
                  response_model=CurrentUserProfile,
                  response_model_exclude_none=True,
                  summary="获取当前登录的管理员-前端用")
async def get_admin_profile(current_user: SysUser = Depends(get_current_user)):
    return {"data": current_user}


@login_router.post("/api/login",
                   summary="教师或学生登录-前端用",
                   tags=["登录认证"],
                   response_model=Token)
async def login_as_user(account: UserLoginInfo,
                        db: AsyncSession = Depends(get_async_db)):

    # 从 redis 中读取 token
    # token_key = f"{account.username}:access-token"
    # access_token = await redis_client.get(token_key)
    access_token = ""
    if not access_token:
        # 查询user是否存在
        crud = crud_teacher if account.utype == 1 else crud_student
        user = await crud.find_one(db, {"username": account.username})

        # 校验密码等
        if not user:
            raise HTTPException(status_code=400, detail="账号或密码错误!")
        elif not (await verify_password(account.password, user.password)):
            raise HTTPException(status_code=400, detail="账号或密码错误!")

        # 生成token
        token_payload = {"sub": user.username, "utype": account.utype}
        access_token_expires = datetime.timedelta(
            minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
        access_token = await security.create_token(
            token_payload, expires_delta=access_token_expires)

        # 把token写入redis，用于校验token有效性
        # await redis_client.set(token_key,
        #                        access_token,
        #                        expire=settings.CACHE_TOKEN_EXPIRE_SECONDS)
    return {
        "data": {
            "access_token": access_token,
            "is_first_login": user.is_first_login
        }
    }


@login_router.delete("/logout",
                     response_model=DetailMixin,
                     response_model_exclude_none=True,
                     tags=["登录认证"],
                     summary="退出-前端用")
async def logout(current_user: SysUser = Depends(get_current_user)):
    # token_key = f"{current_user.username}:access-token"
    # await redis_client.delete(token_key)
    return {"data": None}


@login_router.get("/api/account",
                  tags=["登录认证"],
                  response_model=CurrentUserProfile,
                  response_model_exclude_none=True,
                  summary="获取当前登录的用户-前端用")
async def get_user_profile(
        current_user: Union[Teacher, Student] = Depends(get_current_user)):
    return {"data": current_user}


@login_router.put("/profile",
                  response_model=DetailMixin,
                  response_model_exclude_none=True,
                  tags=["登录认证"],
                  summary="修改密码")
async def edit_profile(
    info: EditPwdInfo,
    db: AsyncSession = Depends(get_async_db),
    current_user: Union[SysUser, Teacher, Student] = Depends(get_current_user)):
    # TODO: 校验验证码

    # 判断旧密码是否相等
    if not (await verify_password(info.oldpwd, current_user.password)):
        raise HTTPException(status_code=400, detail="原始密码错误!")
    else:
        if isinstance(current_user, SysUser):
            crud = crud_admin
        elif isinstance(current_user, Teacher):
            crud = crud_teacher
        else:
            crud = crud_student
        # 更新密码
        delattr(info, "code")
        password = UpdatePassword(password=hashed_password(info.newpwd),
                                  is_first_login=False)
        await crud.update(db, current_user, password)
    return {"data": None}
